package c4.player;

import c4.shared.*;

/**
 * Manages the C4Player UI. Enables and disables user interface components for various
 * client states.
 * @see		C4Player
 */
public class C4PlayerUIManager
{
	/**
	 * Creates C4PlayerUIManager object. Obtains references to all other UI Panes.
	 * @param	uiServ the ServerInfoPane UI
	 * @param	uiPlayer the PlayerInfoPane UI
	 * @param	uiStatus the StatusPane UI
	 * @param	uiColour the ColourPane UI
	 * @param	uiSound the SoundEffectsPane UI
	 * @param	uiControl the ControlPane UI
	 * @param	uiOpponent the OpponentInfoPane UI
	 */
	public C4PlayerUIManager(ServerInfoPane uiServ,PlayerInfoPane uiPlayer,
		StatusPane uiStatus,ColourPane uiColour,SoundEffectsPane uiSound,
		ControlPane uiControl,OpponentInfoPane uiOpponent)
	{
		server=uiServ;
		player=uiPlayer;
		status=uiStatus;
		colour=uiColour;
		sound=uiSound;
		control=uiControl;
		opponent=uiOpponent;
	}

	/**
	 * Sets the UI for a 'Player registered' state.
	 */
	public void setForRegistered()
	{
		server.setEnabled(false);
		player.setEnabled(false);
		colour.setEnabled(true);
		sound.setEnabled(true);
		control.setForRegistered();
		opponent.clear();
		status.setStatus("Registered");
	}

	/**
	 * Sets the UI for a 'Player un-registered' state.
	 */
	public void setForUnregistered()
	{
		server.setEnabled(true);
		player.setEnabled(true);
		player.clear();
		colour.setEnabled(true);
		sound.setEnabled(true);
		control.setForUnregistered();
		opponent.clear();
		status.setStatus("Not Registered");
	}

	/**
	 * Sets the UI for a 'Player requesting for a game' state.
	 */
	public void setForRequesting()
	{
		status.setStatus("Requesting...");
		control.setDisabled();
	}

	/**
	 * Sets the UI for a 'Player playing' state.
	 */
	public void setForPlaying(Player uiOpponent,String uiStatus)
	{
		status.setStatus(uiStatus);
		colour.setEnabled(false);
		sound.setEnabled(false);
		control.setForPlaying();
		opponent.setOpponent(uiOpponent);
	}

	private ServerInfoPane server;
	private PlayerInfoPane player;
	private StatusPane status;
	private ColourPane colour;
	private SoundEffectsPane sound;
	private ControlPane control;
	private OpponentInfoPane opponent;

	private boolean DEBUG=true;
}
