package c4.player;

import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;

/**
 * The Connect 4 game control panel. This panel contains the buttons which are used to register and
 * and unregister a player with the server as well as make requests and terminate games.
 */
public class ControlPane extends JPanel
{
	/**
	 * Creates and displays the buttons. The C4Player is responsible for listening to the events that
	 * are generated by the buttons.
	 * @param	c4Player the object to which the listens to the generated action events.
	 * @see		C4Player
	 * @see 	ActionListener
	 */
	public ControlPane(C4Player c4Player)
	{
		controller=c4Player;
		setLayout(new FlowLayout(FlowLayout.CENTER));

		setPreferredSize(new Dimension(140,288));

		register=new JButton("Register");
		register.setPreferredSize(new Dimension(width,height));
		register.addActionListener(controller);
		register.setActionCommand(controller.REGISTER);
		add(register);

		unregister=new JButton("Un-Register");
		unregister.setPreferredSize(new Dimension(width,height));
		unregister.addActionListener(controller);
		unregister.setActionCommand(controller.UNREGISTER);
		add(unregister);

		play=new JButton("Play");
		play.setPreferredSize(new Dimension(width,height));
		play.addActionListener(controller);
		play.setActionCommand(controller.PLAY);
		add(play);

		finish=new JButton("Finish");
		finish.setPreferredSize(new Dimension(width,height));
		finish.addActionListener(controller);
		finish.setActionCommand(controller.END);
		add(finish);

		player=new JButton("Player Stats");
		player.setPreferredSize(new Dimension(width,height));
		player.addActionListener(controller);
		player.setActionCommand(controller.PLAYER);
		add(player);

		exit=new JButton("Exit");
		exit.setPreferredSize(new Dimension(width,height));
		exit.addActionListener(controller);
		exit.setActionCommand(controller.EXIT);
		add(exit);

   		TitledBorder controlBorder=new TitledBorder("Controls");
   		setBorder(controlBorder);
	}

	/**
	 * Enables and disables the buttons for a player is 'un-registered' state.
	 */
	public void setForUnregistered()
	{
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				register.setEnabled(true);
				unregister.setEnabled(false);
				play.setEnabled(false);
				finish.setEnabled(false);
				player.setEnabled(false);
				exit.setEnabled(true);
			}
		});
	}

	/**
	 * Enables and disables the buttons for a player is 'registered' state.
	 */
	public void setForRegistered()
	{
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				register.setEnabled(false);
				unregister.setEnabled(true);
				play.setEnabled(true);
				finish.setEnabled(false);
				player.setEnabled(true);
				exit.setEnabled(false);
			}
		});
	}

	/**
	 * Enables and disables the buttons for a player is 'playing' state.
	 */
	public void setForPlaying()
	{
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				register.setEnabled(false);
				unregister.setEnabled(false);
				play.setEnabled(false);
				finish.setEnabled(true);
				player.setEnabled(false);
				exit.setEnabled(false);
			}
		});
	}

	/**
	 * Disables all buttons.
	 */
	public void setDisabled()
	{
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				register.setEnabled(false);
				unregister.setEnabled(false);
				play.setEnabled(false);
				finish.setEnabled(false);
				player.setEnabled(false);
				exit.setEnabled(false);
			}
		});
	}

	private C4Player controller;
	private JButton register;
	private JButton unregister;
	private JButton play;
	private JButton finish;
	private JButton player;
	private JButton exit;
	private JRadioButton red;
	private JRadioButton yellow;
	private final static int width=120;
	private final static int height=37;
}
