package c4.player;

import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import java.awt.geom.*;
import java.net.*;
import java.util.*;
import javax.swing.*;
import javax.swing.border.*;
import c4.shared.*;


/**
 * The Connect 4 game visualisation pane. Animates counters falling down slots, checks for game wins
 * losses or draws and checks the validity of a move.
 */
public class GamePane extends JPanel implements Runnable
{
	/**
	 * Creates the GamePane object. Creates anon. inner class Mouse Listeners used to detect moves.
	 * @param	c4Player reference to communicate moves.
	 */
	public GamePane(C4Player c4Player)
	{
		controller=c4Player;
		setPreferredSize(new Dimension(280,252));//was 280,260 was 280,288
   		//TitledBorder gameBorder=new TitledBorder("Connect 4");
   		//setBorder(gameBorder); // can't put border in same panel as paint method

		addMouseMotionListener(new MouseMotionAdapter() {
			public void mouseMoved(MouseEvent e) {
				if (!myMove) return;
				if (moving) return;
				int slot=e.getX();
				if (slot<35)
					mX=-50;
				else if (slot<65)
					mX=23;
				else if (slot<95)
					mX=53;
				else if (slot<125)
					mX=83;
				else if (slot<155)
					mX=113;
				else if (slot<185)
					mX=143;
				else if (slot<215)
					mX=173;
				else if (slot<255)
					mX=203;
				else
					mX=-50;
				mY=counterStartHeight;
				counterColor=playerColor;
				repaint();
			}
		});

		addMouseListener(new MouseAdapter() {
			public void mouseClicked(MouseEvent e) {
				if (!myMove) return;
				int column=0;
				int slot=e.getX();
				if (slot<35)
					return;
				else if (slot<65)
					column=0;
				else if (slot<95)
					column=1;
				else if (slot<125)
					column=2;
				else if (slot<155)
					column=3;
				else if (slot<185)
					column=4;
				else if (slot<215)
					column=5;
				else if (slot<255)
					column=6;
				else
					return;
				Move move=new Move(player,column);
				if (isMoveValid(move))
					controller.updateMoves(move);
			}

			public void mouseEntered(MouseEvent e) {
				setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
				repaint();
			}

			public void mouseExited(MouseEvent e) {
				if (!moving)
					mX=-50;
				setCursor(Cursor.getDefaultCursor());
				repaint();
			}
		});
	}

	/**
	 * Checks to see that a counter slot is not to high to accept another counter.
	 * @param	move the move to be checked.
	 * @return	true if move valid, false otherwise.
	 */
	public boolean isMoveValid(Move move)
	{
		return (slotHeight[move.getColumn()]<ROWS)?true:false;
	}

	/**
	 * Accessor to determine who's move is next.
	 * @return	true if players move (C4Player owner obj.) false if opponent.
	 */
	public boolean isMyMove()
	{
		return myMove;
	}

	/**
	 * Mutator method to set owner of next move.
	 * @param	bool if true then this players move, flase the oppoent.
	 */
	public void setMyMove(boolean bool)
	{
		myMove=bool;
	}

	/**
	 * Obtains the value of the game grid. Game represented by a 2D array. '0' represent an empty slot
	 * position, -1 an opponents counter and 1 the players counter. This method returns the value of any
	 * slot position. If outside the game pane then returns -2.
	 * @param	gRow the row value
	 * @param	gColumn the column value
	 * @param	dDirection, a two element array specifying row and column offsets.
	 */
	public int getGameValue(int gRow,int gColumn,int[] gDirection)
	{
		int row=gRow+gDirection[0];
		int column=gColumn+gDirection[1];
		if (row>-1 && row<ROWS && column>-1 && column<COLUMNS)
			return game[row][column];
		return -2;
	}


	public int getGameValue(int gRow,int gColumn,int[] gDirection,int[][] mGame)
	{
		int row=gRow+gDirection[0];
		int column=gColumn+gDirection[1];
		if (row>-1 && row<ROWS && column>-1 && column<COLUMNS)
			return mGame[row][column];
		return -2;
	}



	/**
	 * Checks to see if move is a winning move.
	 * @return	won, true if move is a winning move false otherwise.
	 */
	public boolean isMoveAWinner(Move move)
	{
		int currentColumn=move.getColumn();
		int currentRow=slotHeight[currentColumn];
		int row,column;
		int count=1;
		boolean won=false;
		boolean emptyORnotsame=false;
		int[] look={0,0};
		if (DEBUG) System.out.println("Checking for Winner");
		for (int i=0;i<7 && !won;++i) {
			count=1;
			look[0]=direction[i][0];
			look[1]=direction[i][1];
			for (int j=0;j<2 && count<=4;++j)
			{
				emptyORnotsame=false;
				column=currentColumn;
				row=currentRow;
				look[0]*=(-1); //look in the opposite direction
				look[1]*=(-1);
				while (!emptyORnotsame && count<=4)
				{
					if (DEBUG && j==1) System.out.println("Looking in opposite direction");
					if (DEBUG) System.out.println("At row=" + row +	" column=" + column);
					if (DEBUG) System.out.println("Looking relative by row plus "
						 + look[0] + " and column plus " + look[1]);
					int nextValue=getGameValue(row,column,look);
					if (nextValue==-2) { //out of the board
						emptyORnotsame=true;
						if (DEBUG) System.out.println("Out of bounds");
					} else if (nextValue==0) { //no counter in this position
						emptyORnotsame=true;
						if (DEBUG) System.out.println("Position empty");
					} else if (nextValue!=game[row][column]) {
						emptyORnotsame=true;
						if (DEBUG) System.out.println("Not the same colour");
					} else { //adjacent counter is same
						row+=look[0];
						column+=look[1];
						++count;
						if (DEBUG) System.out.println("Same colour");
					}
					if (DEBUG) System.out.println("Counters detected in a line so far " + count);
				}
			}
			won=(count>=4)?true:false;
		}
		return won;
	}



	public int weight(Move move,int[][] mGame)
	{
		int currentColumn=move.getColumn();
		int currentRow=slotHeight[currentColumn];
		int row,column;
		int count=1;
		boolean won=false;
		boolean emptyORnotsame=false;
		int[] look={0,0};
		int bestcount=0;
		for (int i=0;i<7;++i) {
			count=1;
			look[0]=direction[i][0];
			look[1]=direction[i][1];
			for (int j=0;j<2;++j)
			{
				emptyORnotsame=false;
				column=currentColumn;
				row=currentRow;
				look[0]*=(-1); //look in the opposite direction
				look[1]*=(-1);
				while (!emptyORnotsame)
				{
					int nextValue=getGameValue(row,column,look,mGame);
					if (nextValue==mGame[row][column]) {
						row+=look[0];
						column+=look[1];
						++count;
					} else {
						emptyORnotsame=true;
					}
					if (count>bestcount) bestcount=count;
				}
			}
		}
		bestcount*=(move.getPlayer()==player)?1:(-1);
		return bestcount;
	}


	/**
	 * Dervives the computers best move. Uses a Minimax search principle and follows algorithm laid
	 * out in 'Artificial Intelligence' by Rich and Knight (McGraw-Hill).
	 * @param	treeNode a temporal representation of the game board and the last move.
	 * @param	depth the cuurent depth of the search, ie the cuurent level of 'look-aheads'.
	 * @param	moveCode either -1 for computer or 1 for player, alternates at each depth level.
	 * @return	a BestMove object containing the move weight and the path.
	 */

	public BestMove minimax(GameTreeNode treeNode,int depth,int moveCode,int MINIMAX_SEARCH_DEPTH)
	{
		// the 'part x' refer to stages of the minmax algorithm as detailed in Rich & Knight
		int value;
		Vector path;
		if (depth>=MINIMAX_SEARCH_DEPTH) {	// part 1
			value=weight(treeNode.getMove(),treeNode.getGame()); // weight==static in R&K
			path=new Vector(3);
			if (DEBUG) System.out.print("   bestscore=" + bestscore);
			if (DEBUG) System.out.print("   depth=" + depth);
			if (DEBUG) System.out.print("   column=" + treeNode.getMove().getColumn());
			if (DEBUG) System.out.println("   moveCode=" + moveCode);
			return new BestMove(value,path);
		}

		//part 2
		Player nextPlayer=(moveCode==1)?player:computer;
		Vector successors=treeNode.moveGenerator(nextPlayer,moveCode);

		// part 3
		if (successors.size()==0) {
			if (DEBUG) System.out.println("No Successors");
			value=weight(treeNode.getMove(),treeNode.getGame());
			path=new Vector(3);
			return new BestMove(value,path);
		}

		//part 4
		if (successors.size()>0) {
			//if (DEBUG) System.out.println("Successors found");
			BestMove resultsucc;
			Player next;
			int newvalue=-99;
			for (int i=0;i<successors.size();++i) {
				GameTreeNode node=(GameTreeNode)successors.elementAt(i);

				// part 4a
				resultsucc=minimax(node,depth+1,moveCode*(-1),MINIMAX_SEARCH_DEPTH);

				// part 4b

				if (MINIMAX_SEARCH_DEPTH%2==0)
					newvalue=resultsucc.getValue();
				else
					newvalue=resultsucc.getValue()*(-1);

				//part 4c
				if (newvalue>bestscore) {
					if (DEBUG) System.out.print("newvalue=" + newvalue);
					bestscore=newvalue;
					bestpath.addElement(new Integer(node.getMove().getColumn()));
				}
			}
		}
		// part 5
		return new BestMove(bestscore,bestpath);
	}


	/**
	 * Move controller method. Strarts move visualisation, waits for move to complete and then calls
	 * isMoveAWinner returning the result to the is C4Player owner.
	 * @return	won, '0' if no winner, '1' if player win, '3' if opponent win and '2' if drawn.
	 */
	public synchronized int move(Move move)
	{
		moving=true;
		if (DEBUG) System.out.println("Moving is " + moving);
		column=move.getColumn();
		counterColor=(myMove)?playerColor:opponentColor;
		mY=counterStartHeight;
		mX=column*30+23;
		moveCompleted=false;
		Thread runner=new Thread(this);
		runner.start();
		try {
			wait();
		} catch(Exception e){}
		game[slotHeight[column]][column]=(myMove)?1:-1;
		int won=0;
		if (myMove) {
			won=(isMoveAWinner(move))?1:0;
		} else {
			won=(isMoveAWinner(move))?3:0;
		}
		if (++movesMade==42 && won==0) won=2;
		++slotHeight[column];
		moving=false;
		if (DEBUG) System.out.println("Moving is " + moving);
		myMove=(myMove)?false:true;
		return won;
	}


	public synchronized int computerMove()
	{
		moving=true;
		if (DEBUG) System.out.println("Moving is " + moving);
		bestscore=-99;
		bestpath.clear();
		GameTreeNode treeNode=new GameTreeNode(game,slotHeight,new Move(computer,0));
		BestMove bestmove=minimax(treeNode,0,-1,3);
		Vector path=bestmove.getPath();
		Integer computerColumn=(Integer)path.elementAt(path.size()-1);
		if (movesMade==0) {
			column=(int)(Math.random()*7);
			System.out.println("First Move=" + column);
		} else {
			column=computerColumn.intValue();
		}
		counterColor=opponentColor;
		mY=counterStartHeight;
		mX=column*30+23;
		moveCompleted=false;
		Thread runner=new Thread(this);
		runner.start();
		try {
			wait();
		} catch(Exception e){}
		game[slotHeight[column]][column]=(myMove)?1:-1;
		int won=0;
		if (myMove) {
			won=(isMoveAWinner(new Move(computer,column)))?1:0;
		} else {
			won=(isMoveAWinner(new Move(computer,column)))?3:0;
		}
		if (++movesMade==42 && won==0) won=2;
		++slotHeight[column];
		moving=false;
		if (DEBUG) System.out.println("Moving is " + moving);
		myMove=(myMove)?false:true;
		return won;
	}



	/**
	 * Counter animation thread. Makes 'callback' to move method through wait() notify()
	 */
	public synchronized void run()
	{
		float delay=100;
		int stop=((ROWS-1)-slotHeight[column])*27+54;
		while (mY<stop) {
			mY+=dY;
			delay*=0.90;
			try {
				Thread.sleep((int)delay);
			} catch(InterruptedException e){}
			repaint();
		}
		notify();
		moveCompleted=true;
	}

	/**
	 * Mutator method to set the players counter colour.
	 * @param	colour the players choosen counter colour
	 */
	public void setColour(Color colour)
	{
		playerColor=colour;
		opponentColor=(playerColor==Color.red)?Color.yellow:Color.red;
	}


	/**
	 * Clears the connect 4 board in preparation fora game.
	 * @param	gPlayer the player.
	 */
	public void reset(Player gPlayer)
	{
		player=gPlayer;
		movesMade=0;
		for(int j=0;j<7;++j){
			slotHeight[j]=0;
			for(int i=0;i<6;++i)
				game[i][j]=0;
		}
		mX=-50;
		repaint();
	}

	/**
	 * Overides the update method. Makes call to paintComponent.
	 */
	public void update(Graphics g)
	{
		paintComponent(g);
	}

	/**
	 * Draws the Game state for the counter animation and game visualisation. Called via repaint()
	 * from run method.
	 */
	public void paintComponent(Graphics g)
	{
		super.paintComponent(g);
		Graphics2D g2=(Graphics2D)g;

		currentBoard=new BufferedImage(280,260,BufferedImage.TYPE_INT_ARGB);
		offScreen=currentBoard.createGraphics();
		ac=AlphaComposite.getInstance(AlphaComposite.SRC,1.0f);
		offScreen.setComposite(ac);
		offScreen.setColor(Color.blue);
		offScreen.fill3DRect(15,50,width,height,true);//was 15,50,width,height,true
		offScreen.fill3DRect(10,49,5,200,true);//was 10,49,5,200,true
		offScreen.fill3DRect(235,49,5,200,true);//was 235,49,5,200,true
		ac=AlphaComposite.getInstance(AlphaComposite.SRC,0.0f);
		offScreen.setComposite(ac);
		offScreen.fill(new Rectangle2D.Double(15,0,width,50));//was 15,0,width,50
		for(int x=25,j=0;x<221;x+=30,++j){
			for(int y=57,i=0;y<193;y+=27,++i){
				offScreen.drawOval(x,y,20,20);
				offScreen.fill(new Ellipse2D.Double(x,y,20,20));
				if (game[(ROWS-1)-i][j]!=0){
					ac=AlphaComposite.getInstance(AlphaComposite.DST_OVER,1.0f);
					offScreen.setComposite(ac);
					if (game[(ROWS-1)-i][j]==-1){
						offScreen.setColor(opponentColor);
					} else {
						offScreen.setColor(playerColor);
					}
					offScreen.fill(new Ellipse2D.Double(x-2,y-2,24,24));
					ac=AlphaComposite.getInstance(AlphaComposite.SRC,0.0f);
					offScreen.setComposite(ac);
				}
			}
		}
		ac=AlphaComposite.getInstance(AlphaComposite.DST_OVER,1.0f);
		offScreen.setComposite(ac);
		offScreen.setColor(counterColor);
		offScreen.fillOval(mX,mY,24,24);
		g2.drawImage(currentBoard,null,15,0);// was 15,25
	}


	private int bestscore=-99;
	private Vector bestpath=new Vector(3);
	private C4Player controller;
	private Player player;
	private Player computer=new Player("Computer",null);
	private BufferedImage currentBoard;
	private Graphics2D offScreen;
	private AlphaComposite ac;
	private int column;
	private int movesMade=0;
	private int mX=-50,mY;
	private Color counterColor;
	private Color counterShade;
	private Color playerColor=Color.red;
	private Color opponentColor=Color.yellow;
	private Color darkRed=new Color(220,0,0);
	private Color darkYellow=new Color(240,240,0);
	private Color darkBlue=new Color(0,0,160);
	private boolean myMove=false;
	private boolean moving=false;
	private boolean moveCompleted=false;
	private boolean DEBUG=false;
	private final static int dY=5;
	private final static int width=220;
	private final static int height=169;
	private final static int MINIMAX_SEARCH_DEPTH=3;
	private final static int counterStartHeight=10;
	private int[] slotHeight=new int[COLUMNS];
	private int[][] game=new int[ROWS][COLUMNS];
	private int[][] direction={{1,-1},{1,1},{1,0},{0,-1},{0,1},{-1,-1},{-1,1}};

	private final static int ROWS=6;
	private final static int COLUMNS=7;
}