package c4.player;


import java.util.Vector;
import c4.shared.*;


public class MinimaxMoveGenerator
{
	public MinimaxMoveGenerator()
	{
		getmoves();
	}

	public BestMove minimax(GameTreeNode treeNode,int depth,int moveCode)
	{
		//int bestscore=-99;
		//Vector bestpath=new Vector();

		//if (DEBUG) System.out.println("Inside minimax, depth=" + depth);
		int value;
		Vector path;
		if (depth>=2) {	// part 1
			value=isMoveAWinner(treeNode.getMove(),treeNode.getGame());
					//if (DEBUG) System.out.print("   bestscore=" + bestscore);
					//if (DEBUG) System.out.print("   depth=" + depth);
					//if (DEBUG) System.out.print("   column=" + treeNode.getMove().getColumn());
					//if (DEBUG) System.out.println("   moveCode=" + moveCode);
			path=new Vector(3);
			return new BestMove(value,path);
		}

		Player nextPlayer=(moveCode==1)?player:computer;
		Vector successors=treeNode.moveGenerator(nextPlayer,moveCode); //part 2

		if (successors.size()==0) { // part 3
			if (DEBUG) System.out.println("No Successors");
			value=isMoveAWinner(treeNode.getMove(),treeNode.getGame());
			path=new Vector(3);
			return new BestMove(value,path);
		}

		if (successors.size()>0) { //part 4

			//bestscore=-99;
			//bestpath=new Vector(3);

			//if (DEBUG) System.out.println("Successors found");
			BestMove resultsucc;
			Player next;
			int newvalue=-99;
			for (int i=0;i<successors.size();++i) {
				GameTreeNode node=(GameTreeNode)successors.elementAt(i);
				resultsucc=minimax(node,depth+1,moveCode*(-1)); // part 4a


				//if (DEBUG) System.out.println("Moves explored=" + (++total));


				//newvalue=resultsucc.getValue()*(-1); // part 4b
				newvalue=resultsucc.getValue();// part 4b

				//node.print();
				//String s=Console.readString();


				if (newvalue>bestscore) { //part 4c
					if (DEBUG) System.out.print("newvalue=" + newvalue);
					bestscore=newvalue;
					//bestpath.insertElementAt(new Integer(node.getMove().getColumn()),0);
					bestpath.addElement(new Integer(node.getMove().getColumn()));
					//if (DEBUG) System.out.println("New value > bestscore, newvalue=" + newvalue);
				}
				//if (DEBUG) System.out.println("Finished one level node");
			}
		}
		//node.print();
		//String s=Console.readString();
		return new BestMove(bestscore,bestpath);
	}


	public void reset()
	{
		for(int j=0;j<7;++j){
			slotHeight[j]=0;
			for(int i=0;i<6;++i)
				game[i][j]=0;
		}
	}


	public void move(Move move)
	{
		if (DEBUG) System.out.println("Making a MOVE");
		column=move.getColumn();
		game[slotHeight[column]][column]=(myMove)?1:-1;
		++slotHeight[column];
		myMove=(myMove)?false:true;
	}

	public int isMoveAWinner(Move move,int[][] mGame)
	{
		int currentColumn=move.getColumn();
		int currentRow=slotHeight[currentColumn];
		int row,column;
		int count=1;
		boolean won=false;
		boolean emptyORnotsame=false;
		int[] look={0,0};
		int bestcount=0;
		//if (DEBUG) System.out.println("Checking for Winner");
		for (int i=0;i<7;++i) {
			count=1;
			look[0]=direction[i][0];
			look[1]=direction[i][1];
			for (int j=0;j<2;++j)
			{
				emptyORnotsame=false;
				column=currentColumn;
				row=currentRow;
				look[0]*=(-1); //look in the opposite direction
				look[1]*=(-1);
				while (!emptyORnotsame)
				{
					//if (DEBUG && j==1) System.out.println("Looking in opposite direction");
					//if (DEBUG) System.out.println("At row=" + row +	" column=" + column);
					//if (DEBUG) System.out.println("Looking relative by row plus "
					//	 + look[0] + " and column plus " + look[1]);
					int nextValue=getGameValue(row,column,look,mGame);
					if (nextValue==-2) { //out of the board
						emptyORnotsame=true;
					//	if (DEBUG) System.out.println("Out of bounds");
					} else if (nextValue==0) { //no counter in this position
						emptyORnotsame=true;
					//	if (DEBUG) System.out.println("Position empty");
					} else if (nextValue!=mGame[row][column]) {
						emptyORnotsame=true;
					//	if (DEBUG) System.out.println("Not the same colour");
					} else { //adjacent counter is same
						row+=look[0];
						column+=look[1];
						++count;
					//	if (DEBUG) System.out.println("Same colour");
					}
					//if (DEBUG) System.out.println("Counters detected in a line so far " + count);
					if (count>bestcount) bestcount=count;
				}
			}
		}
		//if (DEBUG) System.out.println("Count=" + bestcount);
		//count=(move.getPlayer()==player)?count*(-1):count;
		count=(move.getPlayer()==player)?count:count*(-1);
		return bestcount;
	}

	public int getGameValue(int gRow,int gColumn,int[] gDirection,int[][] mGame)
	{
		int row=gRow+gDirection[0];
		int column=gColumn+gDirection[1];
		if (row>-1 && row<ROWS && column>-1 && column<COLUMNS)
			return mGame[row][column];
		return -2;
	}

	public boolean isMyMove()
	{
		return myMove;
	}

	public boolean isMoveValid(Move move)
	{
		return (slotHeight[move.getColumn()]<ROWS)?true:false;
	}

	public void getmoves()
	{
		Move move=new Move(player,-1);
		BestMove bestmove;
		int slot;
		boolean end=false;
		int[][] tempgame=new int[ROWS][COLUMNS];
		int[] tempslotHeight=new int[COLUMNS];
		do {


				total=0;

				bestscore=-99;
				bestpath.clear();
				GameTreeNode treeNode=new GameTreeNode(game,slotHeight,new Move(computer,0));
				bestmove=minimax(treeNode,0,-1);
				if (DEBUG) System.out.println("Computer's move weight is " + bestmove.getValue());

				Vector path=bestmove.getPath();
				//for (int i=0;i<path.size();++i)
				//	if (DEBUG) System.out.println("path " + (Integer)path.elementAt(i));
				Integer computerColumn=(Integer)path.elementAt(path.size()-1);
				move.setColumn(computerColumn.intValue());
				move(move);


		for (int i=ROWS-1;i>=0;--i) {
			for (int j=0;j<COLUMNS;++j) {
				System.out.print("\t" + game[i][j]);
			}
			System.out.println("\n");
		}
		System.out.println("");
		for (int i=0;i<COLUMNS;++i)
			System.out.print("\t" + slotHeight[i]);
		System.out.println("\n");




			slot=Console.readInt("Enter player move");
			if (DEBUG) System.out.println("Slot=" + slot);
			if (slot>-1) {
				move.setColumn(slot);
				move(move); // players move






			} else end=true;
		} while (!end);
	}

	private static void main(String[] args)
	{
		MinimaxMoveGenerator mover=new MinimaxMoveGenerator();
	}

	private int bestscore=-99;
	private Vector bestpath=new Vector(3);


	//private int bestscore=-99;
	//private Vector bestpath=new Vector(3);

	private Player player=new Player("Will","Here");
	private Player computer=new Player("Computer","Cyber");
	private int column;
	private int movesMade=0;
	private boolean myMove=true;
	private boolean DEBUG=true;

	private int total=0;

	private int[] movegen={0,1,2,3,4,5,6};

	private int[] slotHeight=new int[COLUMNS];
	private int[][] game=new int[ROWS][COLUMNS];
	private int[][] direction={{1,-1},{1,1},{1,0},{0,-1},{0,1},{-1,-1},{-1,1}};
	private final static int ROWS=6;
	private final static int COLUMNS=7;

}


