package c4.player;

import javax.swing.*;
import java.awt.*;
import javax.swing.border.*;
import c4.shared.*;

/**
 * UI for the displays of the player's details.
 */
public class PlayerInfoPane extends JPanel
{
	/**
	 * Creates the PlayerInfoPane.
	 */
	public PlayerInfoPane()
	{
		setLayout(new GridBagLayout());
		GridBagConstraints gbc=new GridBagConstraints();
		gbc.fill=GridBagConstraints.HORIZONTAL;
		gbc.anchor=GridBagConstraints.WEST;
		gbc.weightx=0;//100;
		gbc.weighty=0;//100;
		add(name,gbc,0,0,1,1);
		add(nameField,gbc,1,0,3,1);
		add(locat,gbc,4,0,1,1);
		add(locatField,gbc,5,0,3,1);
		add(played,gbc,0,1,1,1);
		add(playedField,gbc,1,1,1,1);
		gbc.fill=GridBagConstraints.NONE;
		gbc.anchor=GridBagConstraints.EAST;
		add(won,gbc,2,1,1,1);
		add(wonField,gbc,3,1,1,1);
		gbc.anchor=GridBagConstraints.WEST;
		add(drawn,gbc,4,1,1,1);
		add(drawnField,gbc,5,1,1,1);
		gbc.anchor=GridBagConstraints.EAST;
		add(lost,gbc,6,1,1,1);
		add(lostField,gbc,7,1,1,1);
		playedField.setEditable(false);
		wonField.setEditable(false);
		drawnField.setEditable(false);
		lostField.setEditable(false);
   		TitledBorder playerInfoBorder=new TitledBorder("Player Details");
   		setBorder(playerInfoBorder);
	}

	/**
	 * Accessor method to return the newly entered Player object.
	 * @return	player the new player object.
	 * @see 	Player
	 */
	public Player getPlayer()
	{
		String name=nameField.getText().trim();
		String locat=locatField.getText().trim();
		Player player=new Player(name,locat);
		return player;
	}

	/**
	 * Updates the UI to relect any changes to the player object game statistics.
	 * @params	pPlayer the new player details.
	 */
	public void updatePlayer(Player p)
	{
		final Player player=p;
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				playedField.setText("" + player.getPlayed());
				wonField.setText("" + player.getWin());
				drawnField.setText("" + player.getDrawn());
				lostField.setText("" + player.getLost());
			}
		});
	}

	/**
	 * Clears the contents of the PlayerInforPane.
	 */
	public void clear()
	{
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				nameField.setText("");
				locatField.setText("");
				playedField.setText("0");
				wonField.setText("0");
				drawnField.setText("0");
				lostField.setText("0");
			}
		});
	}

	/**
	 * Enables and disables the player name and location fields.
	 * @param	bool if true filed are enabled, false disables fileds.
	 */
	 public void setEnabled(boolean bool)
	 {
	 	final boolean enabled=bool;
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				nameField.setEditable(enabled);
				locatField.setEditable(enabled);
			}
		});
	}

	private void add(Component c, GridBagConstraints gbc,int x,int y,int w,int h)
	{
		gbc.gridx=x;
		gbc.gridy=y;
		gbc.gridwidth=w;
		gbc.gridheight=h;
		gbc.weightx=0;
		gbc.weighty=0;
		add(c,gbc);
	}

	/**
	 * Checks to see whether both the players name and location have been entered.
	 * @return PlayerInfoPane.OK if entry is correct, PlayerInfoPane.NAME_FIELD if
	 * a name is not entered and PlayerInfoPane.LOCATION_FIELD if the location is not
	 * entered.
	 */
	public int checkEntry()
	{
		if (DEBUG) System.out.println("Name:" + nameField.getText().trim() +
			"\tLocation:" + locatField.getText().trim());
		if (nameField.getText().trim().equals("")) return this.NAME_FIELD;
		if (locatField.getText().trim().equals("")) return this.LOCATION_FIELD;
		else return this.OK;
	}

	private JTextField nameField=new JTextField(18);
	private JTextField locatField=new JTextField(18);
	private JTextField playedField=new JTextField("0",4);
	private JTextField wonField=new JTextField("0",4);
	private JTextField drawnField=new JTextField("0",4);
	private JTextField lostField=new JTextField("0",4);
	private JLabel name=new JLabel("Name");
	private JLabel locat=new JLabel("Location");
	private JLabel played=new JLabel("Played");
	private JLabel won=new JLabel("Won");
	private JLabel drawn=new JLabel("Drawn");
	private JLabel lost=new JLabel("Lost");

	private boolean DEBUG=false;

	/** Used to signify that data entry conditions have been satisfied.*/
	public static int OK=0;
	/** Used to signify that the player has not entered their name.*/
	public static int NAME_FIELD=1;
	/** Used to signify that the player has not entered their location.*/
	public static int LOCATION_FIELD=2;
}
