package c4.server;

import java.awt.*;
import java.awt.event.*;
import java.rmi.*;
import java.rmi.server.*;
import java.net.*;
import javax.swing.border.*;
import c4.shared.*;


/**
 * The Connect 4 server program creates the majority of the server objects. It also registers all of
 * the remote objects with the RMI Registry through their respective servers objects.
 */
public class C4Server extends Frame implements FocusListener
{
	/**
	 * Creates the server objects.
	 */
	public C4Server()
	{
		addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				System.exit(0);
			}
		});

		servControlPane=new ServControlPane(this);
   		ServRegPlayersPane regPlayers=new ServRegPlayersPane();
   		ServGamesPane gamesPane=new ServGamesPane();
   		TitledBorder regPlayersBorder=new TitledBorder("Registered Players");
   		TitledBorder gamesBorder=new TitledBorder("Live Games");
   		regPlayers.setBorder(regPlayersBorder);
   		gamesPane.setBorder(gamesBorder);

		add(servControlPane,"North");
   		add(regPlayers,"Center");
   		add(gamesPane,"South");
   		System.setSecurityManager(new RMISecurityManager());

		try {
			// _Impl objects a object which implement remote interfaces
			// _Server object manage remote object registration
			regPlayersImpl=new RegPlayersImpl();
		   	regPlayersServer=new RegPlayersServer(regPlayersImpl);
			gameControllerImpl=new GameControllerImpl();
		   	gameControllerServer=new GameControllerServer(gameControllerImpl);
			gameMovesImpl=new GameMovesImpl();
		   	gameMovesServer=new GameMovesServer(gameMovesImpl);

	   		String hostname=InetAddress.getLocalHost().getHostName();
	   		if (DEBUG) System.out.println("Host name is: " + hostname);
	   		regPlayersServer.registerRemoteObject();
	   		gameControllerServer.registerRemoteObject();
	   		gameMovesServer.registerRemoteObject();

			regPlayersImpl.setMaxPlayers(servControlPane.getMaxPlayers());
			gameControllerImpl.setMaxGames(servControlPane.getMaxGames());

		} catch (java.rmi.ConnectException e) {
	   	 	ErrorMessages.fatalError("You must run rmiregistry before starting the C4Server.",e);
		} catch (java.net.UnknownHostException e) {
	    		ErrorMessages.fatalError("Can't get current host.", e);
		} catch (java.rmi.RemoteException e) {
	    		ErrorMessages.fatalError("Can't create the registrar.", e);
		} catch (java.net.MalformedURLException e) {
	    		ErrorMessages.fatalError("Can't bind the registrar.", e);
		} catch (java.io.IOException e) {
	    		ErrorMessages.fatalError("Can't open multicast socket.", e);
		} catch (Exception e) {
	    		ErrorMessages.fatalError("Unknown exception on server startup", e);
		}
   	}

	/**
	 * Not used part of FocusListener interface.
	 */
	public void focusGained(FocusEvent e)
	{
	}

    /**
	 * Receives focus events from ServControlPane. Sets the maximum number of allowable registered
	 * players and aliive games in RegPlayersImpl and GamesControllerImpl.
	 * @param	e the FocusEvent
	 * @see		ServControlPane
	 * @see		RegPlayersimpl
	 * @see		GameControllerImpl
	 */
	public void focusLost(FocusEvent e)
	{
		if (DEBUG) System.out.println("Server parameters have changed");
		regPlayersImpl.setMaxPlayers(servControlPane.getMaxPlayers());
		gameControllerImpl.setMaxGames(servControlPane.getMaxGames());
	}

	/**
	 * Creates and displays the C4Server object.
	 */
	public static void main(String args[])
   	{
   		Frame c4server=new C4Server();
   		//c4server.setSize(500,320);
   		c4server.pack();
   		c4server.setTitle("C4Server");
   		c4server.setVisible(true);
   	}

	private ServControlPane servControlPane;

	private GameControllerServer gameControllerServer;
	private GameMovesServer gameMovesServer;
	private RegPlayersServer regPlayersServer;

	private RegPlayersImpl regPlayersImpl;
	private GameMovesImpl gameMovesImpl;
	private GameControllerImpl gameControllerImpl;

	private boolean DEBUG=true;
}