package c4.server;

import java.util.Vector;
import java.rmi.*;
import java.rmi.server.*;
import c4.shared.*;

/**
 * Used to manage registered players between players and the server. Implements the RegPlayers remote interface.
 * @see		RegPlayers
 * @see		Player
 */
public class RegPlayersImpl extends UnicastRemoteObject implements RegPlayers
{
	/**
	 * Creates GameController object. Also creates a vector to hold the list of registered players.
	 * @exeception	RemoteException
	 */
	public RegPlayersImpl() throws RemoteException
	{
		players=new Vector(10,1);
		try {
			socketGate=new SocketGate();
		} catch (Exception e) {
			System.out.println("RegPlayersImpl " + e);
		}
	}

	/**
	 * Accessor method to retreive the list of registered players.
	 * @return	players the list of registered players.
	 * @exeception	RemoteException
	 */
	public synchronized Vector getRegPlayers()
	{
		return players;
	}

	/**
	 * Registers a player with the C4Server.
	 * @param	player the player to be registered.
	 * @return	ID the players new ID.
	 * @exeception	RemoteException
	 */
	public synchronized int registerPlayer(Player player) throws RemoteException
	{
		if (noOfPlayers>=maxPlayers) {
			return -1;
		} else {
			++noOfPlayers;
			player.setUpdate(Player.ADD_NEW_PLAYER);
			player.setStatus(Player.REGISTERED);
			player.setID(nextID());
			players.addElement(player);
			announceRegPlayers(player);
			return player.getID();
		}
	}

	/**
	 * Re-registers a player with the C4Server.
	 * @param	player the player to be registered.
	 * @param	id the players previous id.
	 * @exeception	RemoteException
	 */
	public synchronized int registerPlayer(Player player,int id) throws RemoteException
	{
		if (noOfPlayers>=maxPlayers) {
			return -1;
		} else {
			player.setUpdate(Player.ADD_NEW_PLAYER);
			player.setStatus(Player.REGISTERED);
			player.setID(id);
			players.addElement(player);
			announceRegPlayers(player);
			++noOfPlayers;
			return 1;
		}
	}

	/**
	 * un-registers a player with the C4Server.
	 * @param	player the player to be registered.
	 * @exeception	RemoteException
	 */
	public synchronized void unregisterPlayer(Player player) throws RemoteException
	{
		int i=findPlayerPosition(player);
		if (i!=-1) {
			--noOfPlayers;
			player.setUpdate(Player.REMOVE_PLAYER);
			players.removeElementAt(i);
			announceRegPlayers(player);
		}
	}

	/**
	 * Updates a players game statisitcs.
	 * @param	player the player to be updated.
	 * @exeception	RemoteException
	 */
	public synchronized void updatePlayer(Player player) throws RemoteException
	{
		int i=findPlayerPosition(player);
		if (i!=-1) {
			Player p=(Player)players.elementAt(i);
			player.setUpdate(Player.UPDATE_PLAYER);
			p.updatePlayer(player);
			p.setStatus(player.getStatus());
			announceRegPlayers(player);
		}
	}

	/**
	 * Mutator method to set the maximum allowable number of registered players.
	 * @param	max the maximum number of players.
	 */
	public synchronized void setMaxPlayers(int max)
	{
		maxPlayers=max;
	}

	private int nextID()
	{
		return ++nextid;
	}

	private void announceRegPlayers(Player player)
	{
		System.out.println("Anouncement");
		socketGate.sendRegPlayers(player);
	}

	private int findPlayerPosition(Player player)
	{
		int i;
		int id=player.getID();
		int size=players.size();
		boolean found=false;
		for (i=0;i<size;i++) {
			if (DEBUG) System.out.println("looking at element " + i + " of " + size);
			Player p=(Player)players.elementAt(i);
			if (p.getID()==id) {
				if (DEBUG) System.out.println("found at " + i);
				found=true;
				return i;
			}
		}
		if (DEBUG && found) System.out.println("Found player position at element " + i);
		return -1;
	}

	private Vector players;
	private SocketGate socketGate;
	private int maxPlayers=0;
	private int noOfPlayers=0;
	private int nextid=0;
	private boolean DEBUG=false;
}