package c4.server;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.border.*;

/**
 * The Connect 4 server control panel. This panel contains the server player and game parameters.
 */
public class ServControlPane extends JPanel
{
	/**
	 * Creates the UI components and registers all listeners with the C4Server.
	 * @params	c4Server carry out event listening.
	 * @see C4Server
	 */
	public ServControlPane(C4Server c4Server)
	{
		controller=c4Server;

		setLayout(new FlowLayout(FlowLayout.LEFT));
		// The limit of players
		add(new Label("Max. Players"));
		maxPlayers=new JTextField("10",10);
		maxPlayers.addFocusListener(controller);
		add(maxPlayers);
		// The limit of simultaneous games
		add(new Label("       Max. Games"));
		maxGames=new JTextField("5",10);
		maxGames.addFocusListener(controller);
		add(maxGames);

   		TitledBorder paramsBorder=new TitledBorder("Server Parameters");
		setBorder(paramsBorder);
	}

	/**
	 * Accessor method to return the maximum permittable registered players.
	 * @return	maximum permittable registered players.
	 */
	public int getMaxPlayers()
	{
		try {
			Integer integer=new Integer(maxPlayers.getText());
			return integer.intValue();
		} catch (NumberFormatException e) {
			maxPlayers.setText("10");
			return 10;
		}
	}

	/**
	 * Accessor method to return the maximum permittable number of games.
	 * @return	maximum permittable games.
	 */
	public int getMaxGames()
	{
		try {
			Integer integer=new Integer(maxGames.getText());
			return integer.intValue();
		} catch (NumberFormatException e) {
			maxGames.setText("5");
			return 5;
		}
	}

	private C4Server controller;
	private JTextField maxPlayers;
	private JTextField maxGames;
}