package c4.server;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.net.*;
import java.io.*;
import java.util.Vector;
import c4.shared.*;

/**
 * UI Component that displays all active games. Implements GameControllerListener from which it receives
 * it's Game updates. Includes a JTable object which uses ServGamesModel to manage the content.
 * @see		ServGamesModel
 * @see		GameControllerListener
 * @see		GameControllerListenerThread
 */
public class ServGamesPane extends JPanel implements GameControllerListener
{
	private JScrollPane scrollPane;
	private JTable gameDetails;
	private ServGamesModel model;

	/**
	 * Creates the ServGamesPane and associated objects. Starts the GameControllerListenerThread.
	 */
    public ServGamesPane()
    {
		model=new ServGamesModel();
       	gameDetails=new JTable(model);
		model.addTableModelListener(gameDetails);
		scrollPane=new JScrollPane(gameDetails);
		setLayout(new GridLayout(1,0));
       	add(scrollPane);
		scrollPane.setPreferredSize(new Dimension(500,95));
       	try {
           		new GameControllerListenerThread(this).start();
       	} catch (java.io.IOException e) {
           		System.out.println("Can't start GameControllerListenerThread");
       	}
    }

	/**
	 * GameControllerListener implemented method, receives game objects to update the display.
	 * @param	game a game object to be updated in accordance with the game's udpate type.
	 * @see		GameControllerListenerThread
	 * @see		GameControllerListener
	 * @see		Game
	 */
	public void updateGames(Game game)
    {
		model.updateGame(game);
    }

	/**
	 * Clears the display.
	 */
    public void clear() {
		model.clear();
    }
}
