package c4.server;

import javax.swing.table.*;
import java.util.Vector;
import c4.shared.*;

/**
 * UI Model for ServRegPlayersPane. Manages the display of the registered players displayed in the ServRegPlayersPane.
 * Extends the AbstractTableModel.
 * @see		ServRegPlayersPane
 * @see		AbstractTableModel
 */
class ServRegPlayersModel extends AbstractTableModel
{
	protected static int NUM_COLUMNS=8;
	protected static int START_NUM_ROWS=6;
	protected int nextEmptyRow=0;
	protected int numRows=0;

	private boolean DEBUG=false;

	/** Column name.*/
	static final public String idName="ID";
	/** Column name.*/
	static final public String playerName="Player Name";
	/** Column name.*/
	static final public String locationName="Location";
	/** Column name.*/
	static final public String playedName="Played";
	/** Column name.*/
	static final public String winName="Won";
	/** Column name.*/
	static final public String drawnName="Drawn";
	/** Column name.*/
	static final public String lostName="Lost";
	/** Column name.*/
	static final public String playingName="Status";

	protected Vector data=null;

	/**
	 * Creates the ServGamesModel object. Creates a vector to store the list of all of the registered players.
	 */
	public ServRegPlayersModel()
	{
        data=new Vector();
    }

	/**
	 * Accessor method implemented from AbstractTableModel, returns the column names.
	 * @return	the column name.
	 * @see AbstractTableModel
	 */
    public String getColumnName(int column)
    {
		switch (column) {
		  	case 0:
	    		return idName;
		  	case 1:
	    		return playerName;
		  	case 2:
	    		return locationName;
		  	case 3:
	    		return playedName;
		  	case 4:
	    		return winName;
		  	case 5:
	    		return drawnName;
		  	case 6:
	    		return lostName;
		  	case 7:
	    		return playingName;
		}
		return "";
    }

	/**
	 * Accessor method implemented from AbstractTableModel, returns the number of columns.
	 * @return	the number of columns.
	 * @see AbstractTableModel
	 */
    public int getColumnCount() {
       	return NUM_COLUMNS;
    }

	/**
	 * Accessor method implemented from AbstractTableModel, returns the number of rows.
	 * @return	the number of rows.
	 * @see AbstractTableModel
	 */
    public int getRowCount()
    {
       	if (numRows<START_NUM_ROWS) {
           		return START_NUM_ROWS;
       	} else {
           		return numRows;
       	}
    }

	/**
	 * Accessor method implemented from AbstractTableModel, returns the object by a specified row and
	 * column.
	 * @return	the object at a specified row and column.
	 * @see AbstractTableModel
	 */
    public Object getValueAt(int row,int column)
    {
		try {
       		Player p=(Player)data.elementAt(row);
       		switch (column) {
       			case 0:
           			return new Integer(p.getID());
       			case 1:
           			return p.getName();
       			case 2:
           			return p.getLocation();
       			case 3:
           			return new Integer(p.getPlayed());
	       		case 4:
           			return new Integer(p.getWin());
       			case 5:
           			return new Integer(p.getDrawn());
       	 		case 6:
              		return new Integer(p.getLost());
       			case 7:
           			return p.getStatusString();
           	}
		} catch (Exception e) {
		}
		return "";
    }

	/**
	 * Receives player objects which are then added or removed from the model.
	 * @param	player a player object to be removed or added in accordance with the player udpate type.
	 * @see		Player
	 */
    public synchronized void updatePlayer(Player player)
    {
    	int update=player.getUpdate();
    	if (DEBUG) System.out.println("Inside updatePlayer, update=" + update);
    	if (update==Player.ADD_NEW_PLAYER) addPlayer(player);
    	if (update==Player.UPDATE_PLAYER) changePlayer(player);
    	if (update==Player.REMOVE_PLAYER) removePlayer(player);
    }

	/**
	 * Adds a player to the model.
	 * @param	player the player to be added.
	 */
    public void addPlayer(Player player)
    {
       	int id=player.getID();
       	Player p=null;
       	int index=-1;
       	boolean found=false;
		boolean addedRow=false;
       	int i=0;
       	while (!found && (i<nextEmptyRow)) {
       		p=(Player)data.elementAt(i);
       		if (p.getID()==id) {
           		found=true;
           		index=i;
       		} else {
           		i++;
       		}
       	}
       	if (found) { //update old player
	   		data.setElementAt(player,index);
       	} else { //add new player
		 	if (numRows<=nextEmptyRow) {
	           	numRows++;
				addedRow=true;
         	}
          		index=nextEmptyRow;
	    		data.addElement(player);
		}
        nextEmptyRow++;
		if (addedRow) {
	    	fireTableRowsInserted(index,index);
			fireTableRowsUpdated(index,index);
		} else {
	    	fireTableRowsUpdated(index,index);
		}
	}

	/**
	 * Removes a player from the model.
	 * @param	player the player to be removed.
	 */
    public void removePlayer(Player player)
    {
       	int id=player.getID();
       	Player p=null;
       	int index=-1;
       	boolean found=false;
       	int i=0;
       	while (!found && (i<nextEmptyRow)) {
       		p=(Player)data.elementAt(i);
       		if (p.getID()==id) {
           		found=true;
           		index=i;
       		} else {
           		i++;
       		}
      	}

       	if (found) {
	   		data.remove(index);
	        numRows--;
	   		fireTableRowsDeleted(index,index);
			fireTableRowsUpdated(index,nextEmptyRow);
       		index=--nextEmptyRow;
		}
    }

	/**
	 * Update the model to reflect a change in a player.
	 * @param	player the player to be updated.
	 */
    public void changePlayer(Player player)
    {
    	if (DEBUG) System.out.println("Inside changePlayer");
       	int id=player.getID();
       	Player p=null;
       	int index=-1;
       	boolean found=false;

       	int i=0;
       	while (!found && (i<nextEmptyRow)) {
       		p=(Player)data.elementAt(i);
       		if (p.getID()==id) {
           		found=true;
           		index=i;
       		} else {
           		i++;
       		}
       	}

       	if (found) {
      		p.updatePlayer(player);
	   		setValueAt(new Integer(player.getPlayed()),index,3);
	   		setValueAt(new Integer(player.getWin()),index,4);
	   		setValueAt(new Integer(player.getDrawn()),index,5);
	   		setValueAt(new Integer(player.getLost()),index,6);
	   		int status=player.getStatus();
	   		if (status==Player.REGISTERED)
	   			setValueAt(new String("Waiting"),index,7);
	   		if (status==Player.REQUESTING)
	   			setValueAt(new String("Requesting"),index,7);
	   		if (status==Player.PLAYING)
	   			setValueAt(new String("Playing"),index,7);
			fireTableRowsUpdated(index,index);
		}
    }

	/**
	 * Clears the model.
	 */
   	public synchronized void clear()
   	{
		int oldNumRows=numRows;
       	numRows=START_NUM_ROWS;
		data.removeAllElements();
       	nextEmptyRow=0;
		if (oldNumRows>START_NUM_ROWS) {
    		fireTableRowsDeleted(START_NUM_ROWS,oldNumRows-1);
		}
		fireTableRowsUpdated(0,START_NUM_ROWS-1);
   	}
}
