package c4.server;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.net.*;
import java.io.*;
import java.util.Vector;
import c4.shared.*;

/**
 * UI Component that displays all of the registered players. Implements RegPlayerListener from which
 * it receives it's player updates. Includes a JTable object which uses ServGamesModel to manage the
 * content.
 * @see		ServRegPlayersModel
 * @see		RegPlayersListener
 * @see		RegPlayersListenerThread
 */
public class ServRegPlayersPane extends JPanel implements RegPlayersListener
{
	private JScrollPane scrollPane;
	private JTable playerDetails;
	private ServRegPlayersModel model;

	/**
	 * Creates the ServGamesPane and associated objects. Starts the GameControllerListenerThread.
	 */
   	public ServRegPlayersPane()
   	{
		model=new ServRegPlayersModel();
       	playerDetails=new JTable(model);
		model.addTableModelListener(playerDetails);
		scrollPane=new JScrollPane(playerDetails);
		setLayout(new GridLayout(1,0));
       	add(scrollPane);
		scrollPane.setPreferredSize(new Dimension(500,130));
       	try {
       		new RegPlayersListenerThread(this).start();
       	} catch (java.io.IOException e) {
       		//PENDING what to do?
       	}
   	}

	/**
	 * RegPlayersListener implemented method, receives player objects to update the display.
	 * @param	player a player object to be updated in accordance with the player's udpate type.
	 * @see		RegPlayersListenerThread
	 * @see		RegPlayersListener
	 * @see		Player
	 */
   	public void updateRegPlayers(Player player)
   	{
		model.updatePlayer(player);
   	}

	/**
	 * Clears the display.
	 */
   	public void clear()
	{
		model.clear();
   	}
}
