package c4.server;

import java.net.*;
import java.util.Vector;
import c4.shared.*;

/**
 * Sends datagram packets through a multicast socket. Implements Contants which define Inetaddresses
 * and Port information.
 */
public class SocketGate implements Constants
{
	/**
	 * Creates a SocketGate object. Creates MulticastSocket.
	 * @exceptions	MulticastSocket
	 */
	public SocketGate() throws java.io.IOException
	{
		socket=new MulticastSocket(Constants.portNumber);
		registeredPlayerListeningGroup=InetAddress.getByName(Constants.RegPlayerListeningGroup);
		gameControllerListeningGroup=InetAddress.getByName(Constants.GameControllerListeningGroup);
		gameMovesListeningGroup=InetAddress.getByName(Constants.GameMovesListeningGroup);
    }

	/**
	 * Sends a game object.
	 * @param	game the game to be sent.
	 */
   	public void sendGame(Game game)
   	{
       	sendBytes(game.getBytes(),gameControllerListeningGroup);
   	}

	/**
	 * Sends a player object.
	 * @param	player the player to be sent.
	 */
   	public void sendRegPlayers(Player player)
   	{
       	sendBytes(player.getBytes(),registeredPlayerListeningGroup);
    }

	/**
	 * Sends a move object.
	 * @param	move the move to be sent.
	 */
    public void sendMove(Move move)
    {
    	sendBytes(move.getBytes(),gameMovesListeningGroup);
    }

    private void sendBytes(byte[] data,InetAddress group)
    {
       	DatagramPacket packet=new DatagramPacket(data,data.length,group,Constants.portNumber);
       	if (DEBUG) System.out.println("Packet size=" + packet.getLength());
       	try {
       		socket.send(packet);
       	} catch (java.io.IOException e) {
       		System.out.println("Couldn't send muliticast packet");
       	}
    }

    private boolean DEBUG=false;
    private InetAddress registeredPlayerListeningGroup;
    private InetAddress gameControllerListeningGroup;
    private InetAddress gameMovesListeningGroup;
    private MulticastSocket socket=null;
}
