package c4.shared;

import java.util.StringTokenizer;
import java.io.*;

/**
 * Used to hold game player and game status information. Consists of two Player objects and a status.
 * @see		Player
 */
public class Game implements Serializable
{
	/**
	 * Creates a Game object.
	 */
	public Game()
	{
	}

	/**
	 * Creates a Game object from a stream of bytes.
	 * @param	bite an array of bytes.
	 */
	public Game(byte[] bite)
	{
		if (DEBUG) System.out.println("Inside public Game(byte[] bite) bite.length is " + bite.length);
		status=(int)bite[0];
		if (DEBUG) System.out.println("status is " + status);
		byte[] gamePlayersBytes=new byte[bite.length-1];
		System.arraycopy(bite,1,gamePlayersBytes,0,gamePlayersBytes.length);
		String gamePlayers=new String(gamePlayersBytes);
		StringTokenizer tokenizer=new StringTokenizer(gamePlayers,delimiter);
		byte[] playerBytes=tokenizer.nextToken().getBytes();
		byte[] opponentBytes=tokenizer.nextToken().getBytes();
		if (DEBUG) System.out.println("o.k. so far about to create player and opponent");
		player=new Player(playerBytes);
		opponent=new Player(opponentBytes);
	}

	/**
	 * Mutator method to set the player and opponent Player objects. Game status is set to 0.
	 * @param	gPlayer a Player object, the player who creates the Game object.
	 * @param	gOpponent the opponent Player object.
	 */
	public void setGame(Player gPlayer,Player gOpponent)
	{
		player=gPlayer;
		opponent=gOpponent;
		status=0;
	}

	/**
	 * An accessor method to retrieve the player Player object. The player is the creator of the object.
	 * @return	player the game creator.
	 */
	public Player getPlayer()
	{
		return player;
	}

	/**
	 * An accessor method to retrieve the opponent Player object. The player is the creator of the object.
	 * @return	opponent the game opponent.
	 */
	public Player getOpponent()
	{
		return opponent;
	}

	/**
	 * Mutator method to set the game status, request, accept, decline, end, finish and busy.
	 * @param	gStatus the new game status.
	 */
	public void setStatus(int gStatus)
	{
		status=gStatus;
	}

	/**
	 * Accessor method to get the game status.
	 * @return	status the game status.
	 */
	public int getStatus()
	{
		return status;
	}


	/**
	 * Determines whether a game is for a particular player or not.
	 * @param	gPlayer the player to test is game is for them.
	 * @return	true if game is for gPlayer, false otherwise.
	 */
	public boolean isGameForMe(Player gPlayer)
	{
		return ((opponent.getID()==gPlayer.getID())?true:false);
	}

	/**
	 * Accessor method to retrieve an array of bytes for a Game object.
	 * @return	gameBytes the array of bytes which represents the game.
	 */
	public byte[] getBytes()
	{
		byte[] statusBytes={(byte)status};
		byte[] playerBytes=player.getBytes();
		byte[] opponentBytes=opponent.getBytes();
		byte[] delimiterBytes=delimiter.getBytes();

		byte[] gameBytes=new byte[statusBytes.length + playerBytes.length + opponentBytes.length + delimiterBytes.length];

		int copyStart=0;
		System.arraycopy(statusBytes,0,gameBytes,copyStart,statusBytes.length);
		System.arraycopy(playerBytes,0,gameBytes,(copyStart+=statusBytes.length),playerBytes.length);
		System.arraycopy(delimiterBytes,0,gameBytes,(copyStart+=playerBytes.length),delimiterBytes.length);
		System.arraycopy(opponentBytes,0,gameBytes,(copyStart+=delimiterBytes.length),opponentBytes.length);

		if (DEBUG) System.out.println("Created byte object for game, gamebytes.length is " + gameBytes.length);

		return gameBytes;
	}

	/** Game status to signify a game request.*/
	public static int REQUEST=1;
	/** Game status to signify that a game has been accepted.*/
	public static int ACCEPT=2;
	/** Game status to signify that a game has been rejected.*/
	public static int DECLINE=3;
	/** Game status to signify that a game has ended prematurely.*/
	public static int END=4;
	/** Game status to signify that a game has been won, drawn or lost.*/
	public static int FINISHED=6;
	/** Game status to signify a game request was not answered because opponent was busy.*/
	public static int BUSY=5;

	private boolean DEBUG=false;

	private Player player;
	private Player opponent;
	private int status=0;
	private String delimiter=new String("|");
}