package c4.shared;

import java.net.*;
import java.io.*;

/**
 * Listens for incomming Game packets. Sent game packets contain game requests, accepts, declines, ends
 * and finishes. Packets sent by C4Server through GameControllerImpl. Listening address and port specified
 * in Contants.
 * @see		ListenerThread
 * @see		Constants
 * @see		GameControllerImpl
 * @see		Game
 */
public class GameControllerListenerThread extends ListenerThread
{
		/**
		 * Creates a GameControllerListenerThread. Argument notifyee is the object which is informed of
		 * a newly arrived packet.
		 * @param	notifyee the object which is informed of the new Game packet.
		 * @exception	IOException
		 */
    	public GameControllerListenerThread(GameControllerListener notifyee) throws IOException
    	{
			super(Constants.GameControllerListeningGroup);
			this.notifyee=notifyee;
    	}

    	/**
		 * Listens for and rebuilds datagram packet, rebuilds Game object and sends to notifyee.
		 */
		public synchronized void run()
    	{
			DatagramPacket packet;
        	while (stopListening==false) {
	    		byte[] buf=new byte[256];
            	packet=new DatagramPacket(buf,256);
	    		try {
                	socket.receive(packet);
                	int length=packet.getLength();
					if (DEBUG) System.out.println("Game packet size is " + length);
					byte[] rcvd=packet.getData();
					byte[] clean=new byte[length];
					System.arraycopy(rcvd,0,clean,0,length);
					Game update=new Game(clean);
		       		notifyee.updateGames(update);
	    		} catch (IOException e) {
		    		System.out.println("GameControllerListenerThread Except." + e);
	    		}
        	}
    	}

    	private boolean DEBUG=false;
    	private GameControllerListener notifyee;
}
