package c4.shared;

import java.net.*;
import java.io.*;

/**
 * Listens for incomming Move packets. Packets are sent by C4Server through GameMovesImpl. Listening
 * address and port specified in Contants.
 * @see		ListenerThread
 * @see		Constants
 * @see		GameMovesImpl
 * @see		Move
 */
public class GameMovesListenerThread extends ListenerThread
{
		/**
		 * Creates a GameMovesListenerThread. Argument notifyee is the object which is informed of
		 * a newly arrived packet.
		 * @param	notifyee the object which is informed of the new Move packet.
		 * @exception	IOException
		 */
    	public GameMovesListenerThread(GameMovesListener notifyee) throws IOException
    	{
			super(Constants.GameMovesListeningGroup);
			this.notifyee=notifyee;
    	}

    	/**
		 * Listens for and rebuilds datagram packet, rebuilds Move object and sends to notifyee.
		 */
    	public synchronized void run()
    	{
			DatagramPacket packet;
        	while (stopListening==false) {
	    		byte[] buf=new byte[256];
            	packet=new DatagramPacket(buf,256);
	    		try {
                	socket.receive(packet);
                	int length=packet.getLength();
					if (DEBUG) System.out.println("Game packet size is " + length);
					byte[] rcvd=packet.getData();
					byte[] clean=new byte[length];
					System.arraycopy(rcvd,0,clean,0,length);
					Move update=new Move(clean);
		       		notifyee.updateMoves(update);
	    		} catch (IOException e) {
		    		System.out.println("GameMovesListenerThread Except." + e);
	    		}
        	}
    	}

    	private boolean DEBUG=false;
    	private GameMovesListener notifyee;
}
