package c4.shared;

import java.io.Serializable;

/**
 * Used to hold a players move information. Consists of a Player object and a integer column, the
 * slot that the counter was placed in.
 * @see		Player
 */
public class Move implements Serializable
{
	/**
	 * Create a Move object.
	 * @param	mPlayer the player who made the move.
	 * @param	mColumn the slot in the board that the counter was placed.
	 */
	public Move(Player p,int c)
	{
		player=p;
		column=c;
	}

	/**
	 * Creates a Move object from a stream of bytes.
	 * @param	bite an array of bytes.
	 */
	public Move(byte[] bite)
	{
		column=(int)bite[0];
		byte[] playerBytes=new byte[bite.length-1];
		System.arraycopy(bite,1,playerBytes,0,playerBytes.length);
		player=new Player(playerBytes);
	}

	/**
	 * Accessor method to retrieve the player who made the move.
	 * @return	player the player who made the move.
	 */
	public Player getPlayer()
	{
		return player;
	}

	/**
	 * Mutator method to set the player who made the move.
	 * @param	mPlayer the player of the move.
	 */
	public void setPlayer(Player mPlayer)
	{
		player=mPlayer;
	}

	/**
	 * Accessor method to retrieve the column which the counter was placed.
	 * @return	column the slot which the counter was placed for the move.
	 */
	public int getColumn()
	{
		return column;
	}

	/**
	 * Mutator method to set the column which for the counter move.
	 * @param	mColumn the column of the move.
	 */
	public void setColumn(int mColumn)
	{
		column=mColumn;
	}

	public boolean isMoveForMe(Player player,Player opponent)
	{
		int id=getPlayer().getID();
		return (id==player.getID() || id==opponent.getID())?true:false;
	}

	/**
	 * Accessor method to retrieve an array of bytes for a Move object.
	 * @return	moveBytes the array of bytes which represents the move.
	 */
	public 	byte[] getBytes()
	{
		byte[] columnBytes={(byte)column};
		byte[] playerBytes=player.getBytes();

		byte[] moveBytes=new byte[columnBytes.length + playerBytes.length];

		System.arraycopy(columnBytes,0,moveBytes,0,columnBytes.length);
		System.arraycopy(playerBytes,0,moveBytes,columnBytes.length,playerBytes.length);
		return moveBytes;
	}

	private Player player;
	private int column;
}
