package c4.shared;

import java.io.Serializable;
import java.util.StringTokenizer;

/**
 * Holds all player details.
 */
public class Player implements Serializable
{
	/**
	 * Creates a Player object.
	 * @param	pName the name of the player.
	 */
	public Player(String pName)
	{
		name=pName;
	}

	/**
	 * Creates a Player object with a specified name and location.
	 * @param	pName the name of the player.
	 * @param	pLocation the locatiob of the player.
	 */
	public Player(String pName,String pLocation)
	{
		name=pName;
		location=pLocation;
	}

	/**
	 * Creates a Player object from an array of bytes.
	 * @param	bite an array of bytes representing the player object.
	 */
	public Player(byte[] bite)
	{
		update=(int)bite[0];
		id=(int)bite[1];
		status=(int)bite[2];
		played=(int)bite[3];
		win=(int)bite[4];
		draw=(int)bite[5];
		lost=(int)bite[6];
		byte[] nameAndlocatBytes=new byte[bite.length-7];
		System.arraycopy(bite,7,nameAndlocatBytes,0,nameAndlocatBytes.length);
		String nameAndlocat=new String(nameAndlocatBytes);
		StringTokenizer tokenizer=new StringTokenizer(nameAndlocat,delimiter);
		name=tokenizer.nextToken();
		location=tokenizer.nextToken();
	}

	/**
	 * Resets the players game statisitics. The players played, won, drawn and lost are all set to 0.
	 */
	public void reset()
	{
		if (DEBUG) System.out.println("Inside resetPlayer()");
		played=0;
		win=0;
		lost=0;
		draw=0;
	}

	/**
	 * Update a players game statistics to that of the argument pPlayer.
	 * @param	pPlayer the player objects new statistics.
	 */
	public void updatePlayer(Player pPlayer)
	{
		status=pPlayer.getStatus();
		played=pPlayer.getPlayed();
		win=pPlayer.getWin();
		draw=pPlayer.getDrawn();
		lost=pPlayer.getLost();
	}

	/**
	 * Mutator method to set the players status.
	 * @param	pStatus the new status for the player.
	 */
	public void setStatus(int pStatus)
	{
		status=pStatus;
	}

	/**
	 * Accessor method to get the players status.
	 * @return	the players status.
	 */
	public int getStatus()
	{
		return status;
	}

	/**
	 * Accessor method to get the players status as a meaningful string.
	 * @return	the players status as a String.
	 */
	public String getStatusString()
	{
		if (status==Player.REGISTERED)
			return new String("Waiting");
	    if (status==Player.REQUESTING)
	    		return new String("Requesting");
	    if (status==Player.PLAYING)
	    		return new String("Playing");
	    else return new String("Don't know");
	}

	/**
	 * Mutator method to set the players name.
	 * @param	pName the players new name.
	 */
	public void setName(String pName)
	{
		name=pName;
	}

	/**
	 * Accessor method to get the players name.
	 * @return	name the players name.
	 */
	public String getName()
	{
		return name;
	}

	/**
	 * Accessor method to get the players location.
	 * @return	location the players location.
	 */
	public String getLocation()
	{
		return location;
	}

	/**
	 * Mutator method to set the players location.
	 * @param	pLocation the players new location.
	 */
	public void setLocation(String pLocation)
	{
		location=pLocation;
	}

	/**
	 * Mutator method to set the players update type.
	 * @param	pUpdate the players new update type.
	 */
	public void setUpdate(int pUpdate)
	{
		update=pUpdate;
	}

	/**
	 * Accessor method to get the players update type.
	 * @return	update the players update type.
	 */
	public int getUpdate()
	{
		return update;
	}

	/**
	 * Mutator method to set the players ID.
	 * @param	pID the players new ID.
	 */
	public void setID(int pID)
	{
		id=pID;
	}

	/**
	 * Accessor method to get the players ID.
	 * @return	id the players ID.
	 */
	public int getID()
	{
		return id;
	}

	/**
	 * Mutator method to set the players playing flag.
	 * @param	bool the players new playing flag status.
	 */
	public void setPlaying(boolean bool)
	{
		playing=bool;
	}

	/**
	 * Mutator method to get the players playing flag.
	 * @return	true if player's playing flag is set, else false.
	 */
	public boolean isPlaying()
	{
		return playing;
	}

	/**
	 * Mutator method to set the players myMove flag.
	 * @param	bool the players new myMove flag status.
	 */
	public void setMyMove(boolean bool)
	{
		myMove=bool;
	}

	/**
	 * Mutator method to get the players myMove flag.
	 * @return	true if player's myMove flag is set, else false.
	 */
	public boolean isMyMove()
	{
		return myMove;
	}

	/**
	 * Increments the players played statistic.
	 */
	public void incPlayed()
	{
		++played;
	}

	/**
	 * Accessor method to get the players played statistic.
	 * @return	played the players played statistic.
	 */
	public int getPlayed()
	{
		return played;
	}

	/**
	 * Increments the players win statistic.
	 */
	public void incWin()
	{
		++win;
	}

	/**
	 * Accessor method to get the players win statistic.
	 * @return	win the players win statistic.
	 */
	public int getWin()
	{
		return win;
	}

	/**
	 * Increments the players lost statistic.
	 */
	public void incLost()
	{
		++lost;
	}

	/**
	 * Accessor method to get the players lost statistic.
	 * @return	lost the players lost statistic.
	 */
	public int getLost()
	{
		return lost;
	}

	/**
	 * Increments the players draw statistic.
	 */
	public void incDrawn()
	{
		++draw;
	}

	/**
	 * Accessor method to get the players draw statistic.
	 * @return	draw the players draw statistic.
	 */
	public int getDrawn()
	{
		return draw;
	}

	/**
	 * Accessor method to retrieve an array of bytes for a Player object.
	 * @return	playerBytes the array of bytes which represents the player.
	 */
	public 	byte[] getBytes()
	{
		byte[] numbers={(byte)update,(byte)id,(byte)status,(byte)played,(byte)win,(byte)draw,(byte)lost};
		String playerStrings=new String(name + delimiter + location);
		byte[] playerStringBytes=playerStrings.getBytes();

		byte[] playerBytes=new byte[numbers.length + playerStringBytes.length];

		System.arraycopy(numbers,0,playerBytes,0,numbers.length);
		System.arraycopy(playerStringBytes,0,playerBytes,numbers.length,playerStringBytes.length);
		return playerBytes;
	}

	private boolean DEBUG=false;
	private String name=null;
	private String location=null;
	private int id=-1;
	private int status=0;
	private boolean playing=false;
	private boolean myMove=true;
	private int update=ADD_NEW_PLAYER;
	private int played=0;
	private int win=0;
	private int lost=0;
	private int draw=0;
	private String delimiter=new String("~");

	/** Player status to signify that the player is not registered.*/
	public static int UNREGISTERED=0;
	/** Player status to signify that the player is registered.*/
	public static int REGISTERED=1;
	/** Player status to signify that the player is requesting.*/
	public static int REQUESTING=2;
	/** Player status to signify that the player is playing.*/
	public static int PLAYING=3;
	/** Player status to signify that the player has received a request to play.*/
	public static int RECEIVED_REQUEST=4;
	/** Player update status to signify that the player is to be added to a registered player display.*/
	public static int ADD_NEW_PLAYER=5;
	/** Player update status to signify that the player is to be updated in a registered player display.*/
	public static int UPDATE_PLAYER=6;
	/** Player update status to signify that the player is to be removed fromo a registered player display.*/
	public static int REMOVE_PLAYER=7;

}


