package c4.shared;

import java.net.*;
import java.io.*;

/**
 * Listens for incomming Player packets. Packets are sent by C4Server through RegPlayersImpl. Listening
 * address and port specified in Contants.
 * @see		ListenerThread
 * @see		Constants
 * @see		RegPlayersImpl
 * @see		Player
 */
public class RegPlayersListenerThread extends ListenerThread
{
		/**
		 * Creates a RegPlayersListenerThread. Argument notifyee is the object which is informed of
		 * a newly arrived packet.
		 * @param	notifyee the object which is informed of the new Move packet.
		 * @exception	IOException
		 */
    	public RegPlayersListenerThread(RegPlayersListener notifyee) throws IOException
    	{
			super(Constants.RegPlayerListeningGroup);
			this.notifyee=notifyee;
    	}

    	/**
		 * Listens for and rebuilds datagram packet, rebuilds Player object and sends to notifyee.
		 */
    	public synchronized void run()
    	{
			DatagramPacket packet;
        	while (stopListening==false) {
	    		byte[] buf=new byte[256];
            	packet=new DatagramPacket(buf,256);
	    		try {
                	socket.receive(packet);
                	int length=packet.getLength();
					if (DEBUG) System.out.println("Player packet size is " + length);
					byte[] rcvd=packet.getData();
					byte[] clean=new byte[length];
					System.arraycopy(rcvd,0,clean,0,length);
					Player update=new Player(clean);
		       		notifyee.updateRegPlayers(update);
	    		} catch (IOException e) {
		    		System.out.println("PlayerListenerThread Except." + e);
	    		}
        	}
    	}

    	private RegPlayersListener notifyee;
    	private boolean DEBUG=false;
}
